#include "interrupt.h"
#include "io.h"
#include "utils.h"
#include "sched.h"
#include "kernel.h"

// 中断描述符结构
struct idt_entry {
    uint16_t base_low;
    uint16_t selector;
    uint8_t zero;
    uint8_t type_attr;
    uint16_t base_high;
} __attribute__((packed));

struct idt_ptr {
    uint16_t limit;
    uint32_t base;
} __attribute__((packed));

// 全局变量
static struct idt_entry idt[256] = {0};
static struct idt_ptr idtp;
static volatile uint32_t tick_count = 0;

// 外部汇编函数声明
extern void load_idt(struct idt_ptr* ptr);
extern void timer_isr(void);
extern int clock_command;

// PIC初始化
static void init_pic(void) {
    // 保存masks
    // uint8_t mask1 = inb(PIC1_DATA);
    // uint8_t mask2 = inb(PIC2_DATA);

    // 初始化主PIC
    outb(PIC1_COMMAND, 0x11);  io_wait();  // 初始化命令
    outb(PIC1_DATA, 32);       io_wait();  // 起始中断向量号 = 32
    outb(PIC1_DATA, 0x04);     io_wait();  // 告诉主PIC从PIC连接在IRQ2
    outb(PIC1_DATA, 0x01);     io_wait();  // 8086模式

    // 初始化从PIC
    outb(PIC2_COMMAND, 0x11);  io_wait();  // 初始化命令
    outb(PIC2_DATA, 40);       io_wait();  // 起始中断向量号 = 40
    outb(PIC2_DATA, 0x02);     io_wait();  // 告诉从PIC它在主PIC的IRQ2上
    outb(PIC2_DATA, 0x01);     io_wait();  // 8086模式

    // 恢复masks
    outb(PIC1_DATA, 0xfe);    io_wait();
    outb(PIC2_DATA, 0xff);    io_wait();
}

// 设置中断门
static void set_idt_gate(uint8_t num, uint32_t base) {
    idt[num].base_low = base & 0xFFFF;
    idt[num].base_high = (base >> 16) & 0xFFFF;
    idt[num].selector = 0x08;  // 代码段选择子
    idt[num].zero = 0;
    idt[num].type_attr = 0x8E; // Present=1, DPL=0, Type=14 (32位中断门)
}

// 初始化PIT
static void init_pit(void) {
    uint32_t divisor = TIMER_DIVISOR;
    
    outb(PIT_COMMAND_PORT, 0x36);    // 通道0，方式3，二进制
    io_wait();
    outb(PIT_CHANNEL_0_PORT, divisor & 0xFF);     // 低字节
    io_wait();
    outb(PIT_CHANNEL_0_PORT, (divisor >> 8) & 0xFF); // 高字节
    io_wait();
}

// 定时器中断处理函数
void timer_handler(void) {
    tick_count++;
    
    // 每100个tick (1秒) 输出一次
    if (tick_count % TIMER_FREQ == 0) {
        if(clock_command > 0){
            print_string("Counter:");
            print_int(clock_command);
            print_string("\n");
            clock_command--;
        }
        timer_tick();
    }
    // 发送EOI
    outb(PIC1_COMMAND, 0x20);
}

// 初始化中断系统
void init_interrupts(void) {
    // 设置IDT指针
    idtp.limit = sizeof(idt) - 1;
    idtp.base = (uint32_t)&idt;

    // 初始化PIC
    init_pic();
    
    // 设置定时器中断
    set_idt_gate(IRQ0, (uint32_t)timer_isr);
    
    // 初始化PIT
    init_pit();
    
    // 加载IDT
    load_idt(&idtp);
    
    // 启用定时器中断
    outb(PIC1_DATA, inb(PIC1_DATA) & ~(1 << 0));
    
    // 启用中断
    enable_interrupts();
}

void enable_interrupts(void) {
    asm volatile("sti");
}

void disable_interrupts(void) {
    asm volatile("cli");
}