#include "syscall.h"
#include "video.h" // For printing to the screen
#include "types.h" // For types like `size_t`
#include "string.h"
#include "memory.h" // For memory allocation and free
#include "io.h"     // For inb() and outb() (if needed)
#include "interrupt.h"
#include "sched.h"
#include "kernel.h"

// Global tick counter
static unsigned int system_ticks = 0;

unsigned int eax_val;
unsigned int ebx_val;
unsigned int ecx_val;

// Syscall Handler
void syscall_handler(void)
{
    unsigned int syscall_num;
    unsigned int arg1, arg2;

    // 使用内联汇编读取寄存器值
    asm volatile("mov %%eax, %0" : "=r"(syscall_num));
    asm volatile("mov %%ebx, %0" : "=r"(arg1));
    asm volatile("mov %%ecx, %0" : "=r"(arg2));

    switch (syscall_num)
    {
    case SYSCALL_WRITE:
        syscall_write((const char *)arg1);
        break;
    case SYSCALL_EXIT:
        syscall_exit((int)arg1);
        break;
    case SYSCALL_READ:
        syscall_read((char *)arg1, (size_t)arg2);
        break;
    case SYSCALL_ALLOC:
        // 直接返回分配的内存地址
        {
            void *ptr = syscall_alloc(arg1);
            asm volatile("mov %0, %%eax" : : "r"(ptr));
        }
        break;
    case SYSCALL_FREE:
        syscall_free((void *)arg1);
        break;
    case SYSCALL_SLEEP:
        syscall_sleep((int)arg1);
        break;
    default:
        print_string("Unknown syscall number!\n");
        break;
    }
}

void syscall_write(const char *str)
{
    print_string(str);
}

void syscall_exit(int status)
{
    char status_msg[16];
    itoa(status, status_msg, 10); // 将状态码转换为字符串
    print_string("Exit status: ");
    print_string(status_msg);
    print_string("\n");

    // 终止当前进程的逻辑
    block_current_thread(); // 将当前线程状态设置为阻塞状态
    schedule();                           // 调用调度器切换到其他线程
}

void syscall_read(char *buffer, size_t size)
{
    get_input(buffer, size);
}

// The "alloc" system call - allocate memory
void *syscall_alloc()
{
    return alloc_page();
}

// The "free" system call - free allocated memory
void syscall_free(void *ptr)
{
    free_page(ptr);
}

// The "sleep" system call - put the system to sleep for a certain number of milliseconds
void syscall_sleep(int milliseconds)
{
    unsigned int start_ticks = get_system_ticks();
    unsigned int end_ticks = start_ticks + (milliseconds / 55); // Assuming 55 ms per tick

    // Wait until the number of system ticks has passed
    while (get_system_ticks() < end_ticks)
        ;
}

// Get the current system tick count
unsigned int get_system_ticks(void)
{
    return system_ticks;
}